// Copyright (C) 2005-2006 Satoshi Eguchi. All rights reserved.

#include "StdAfx.h"
#include ".\texfunctiontype.h"
#include <new>

using namespace std;

CTeXFunctionType::CTeXFunctionType(void) : m_ParamCount(0), m_pParams(NULL), m_pSubParam(NULL)
{
	m_FunctionName.Empty();
}

CTeXFunctionType::CTeXFunctionType(LPCTSTR funcname) : m_ParamCount(0), m_pParams(NULL), m_pSubParam(NULL)
{
	m_FunctionName.Empty();
	SetFunctionName(funcname);
}

CTeXFunctionType::CTeXFunctionType(const CString &funcname) : m_ParamCount(0), m_pParams(NULL), m_pSubParam(NULL)
{
	m_FunctionName.Empty();
	SetFunctionName(funcname);
}

CTeXFunctionType::CTeXFunctionType(const CTeXFunctionType &obj) : m_ParamCount(0), m_pParams(NULL), m_pSubParam(NULL)
{
	m_FunctionName.Empty();
	Copy(obj);
}

CTeXFunctionType::~CTeXFunctionType(void)
{
	if (m_pParams) {
		delete [] m_pParams;
	}
	if (m_pSubParam) {
		delete m_pSubParam;
	}
	m_ParamCount = 0;
	m_pParams = NULL;
	m_pSubParam = NULL;
}

BOOL CTeXFunctionType::Copy(const CTeXFunctionType &obj)
{
	m_FunctionName = obj.m_FunctionName;

	if (m_pParams) {	// U
		delete [] m_pParams;
	}
	if (m_pSubParam) {
		delete m_pSubParam;
	}
	m_ParamCount = 0;
	m_pParams = NULL;
	m_pSubParam = NULL;

	if (obj.m_ParamCount) {
		try {
			m_pParams = new CStringArray[obj.m_ParamCount];
		} catch (bad_alloc xa) {
			m_pParams = NULL;
			return FALSE;
		}
		m_ParamCount = obj.m_ParamCount;
		for (UINT i = 0; i < m_ParamCount; i++) {
			m_pParams[i].RemoveAll();
			m_pParams[i].Copy(obj.m_pParams[i]);
		}
	}

	if (obj.m_pSubParam) {
		try {
			m_pSubParam = new CStringArray;
		} catch (bad_alloc xa) {
			m_pSubParam = NULL;
			return FALSE;
		}
		m_pSubParam->RemoveAll();
		m_pSubParam->Copy(*(obj.m_pSubParam));
	}
	
	return TRUE;
}

BOOL CTeXFunctionType::SetParamCount(UINT count)
{
	if (count <= m_ParamCount) {	// p[^͖̐̂
		return TRUE;
	}

	CStringArray *p = NULL;
	try {
		p = new CStringArray[count];
	} catch (bad_alloc xa) {
		return FALSE;
	}
	for (UINT i = 0; i < count; i++) {
		// Ô߁Amۂz
		p[i].RemoveAll();
	}

	if (m_ParamCount) {		// Ɋ蓖ĂĂꍇ
		// ȑO̓eRs[
		for (UINT i = 0; i < m_ParamCount; i++) {
			p[i].Copy(m_pParams[i]);
		}
		delete [] m_pParams;
	}
	m_pParams = p;
	m_ParamCount = count;

	return TRUE;
}

BOOL CTeXFunctionType::EnableSubParam(BOOL enable)
{
	if (!enable) {
		if (m_pSubParam) {
			delete m_pSubParam;
		}
		m_pSubParam = NULL;
		return TRUE;
	}
	if (m_pSubParam) {	// ɗLɂȂĂ
		return TRUE;
	}

	// VK蓖
	try {
		m_pSubParam = new CStringArray;
	} catch (bad_alloc xa) {
		m_pSubParam = NULL;
		return FALSE;
	}
	m_pSubParam->RemoveAll();
	
	return TRUE;
}
